/*
    LP: Laplacian Pyramid operations

    05/01/97 JSP Creation
*/

#ifndef LP_H
#define LP_H

class LP
{
public:
    LP (unsigned width, 
        unsigned height, 
        const unsigned char *image,
        unsigned levels,
        unsigned *bpp);
    LP (const char *filename);
    ~LP ();
    void Compress ();
    void Quantize ();
    void Unquantize ();
    void Decompress ();
    const unsigned char *GetRImage (unsigned level);
    const unsigned char *GetLImage (unsigned level);
    void GetDimensions (unsigned level, 
        unsigned &width, 
        unsigned &height);
    double Entropy (int quantized = 0);
    void Save (const char *filename);

private:
    void AllocateLevels ();
    void Reduce (unsigned level);
    void Expand (unsigned level);
    void Subtract (unsigned level);
    void Add (unsigned level);
    unsigned char Reduce3x3 (const unsigned char *block, unsigned width);
    unsigned char Reduce3x2 (const unsigned char *block, unsigned width);
    unsigned char Reduce2x3 (const unsigned char *block, unsigned width);
    unsigned char Reduce2x2 (const unsigned char *block, unsigned width);
    void Expand3x3 (unsigned char val, unsigned char *block, unsigned width);
    void Expand3x2 (unsigned char val, unsigned char *block, unsigned width);
    void Expand2x3 (unsigned char val, unsigned char *block, unsigned width);
    void Expand2x2 (unsigned char val, unsigned char *block, unsigned width);

    unsigned _width;
    unsigned _height;
    unsigned _levels;
    unsigned char **_r_images;
    unsigned char **_l_images;
    unsigned *_bpp;
};

#endif