/*
    Test: Compress a bunch of Laplcian pyramids with
    random dimensions to make sure Reduce() and 
    Expand() stay within their bounds.

    CRTDBG code is Microsoft v4.2 specific.

    05/01/97 JSP Creation
*/

#include <crtdbg.h>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lp.h"

// MS doesn't define this struct in the ndebug block
// of crtdbg.h

#ifdef NDEBUG
#pragma warning (disable : 4101)
struct _CrtMemState {};
#endif

enum { PASSES = 10 };

main (int argc, char **argv)
{
    try
    {
        if (argc > 3)
            throw "usage: test [passes] [srand_seed]";

        unsigned passes = PASSES;

        if (argc > 1)
            passes = atoi (argv[1]);
        
        cout << "passes: " << passes << endl;

        // Don't seed srand with time because we want to
        // be able to reproduce the test if an error
        // occurs

        unsigned seed = 0;

        if (argc > 2)
            seed = atoi (argv[2]);

        srand (seed);

        cout << "seed: " << seed << endl << endl;

        for (unsigned pass = 0; pass < passes; pass++)
        {
            _CrtMemState ms1;
            _CrtMemCheckpoint (&ms1);

            // Memory checker checks for balanced memory in the
            // next block
            {
                unsigned x = rand () % 1024 + 1;
                unsigned y = rand () % 1024 + 1;
                unsigned char *image = new unsigned char [x * y];
                unsigned levels = rand () % 10 + 1;
                unsigned *bpp = new unsigned [levels];

                for (unsigned index = 0; index < levels; index++)
                    bpp[index] = 8;

                LP lp (x,
                    y,
                    image,
                    levels,
                    bpp);

                delete [] bpp;

                lp.Compress ();

                cout << pass << ": " << 
                    x << "x" << 
                    y << " " << 
                    levels << " levels" << endl;

                if (!_CrtCheckMemory ())
                    throw "The heap is corrupt";

                delete [] image;
            }

            _CrtMemState ms2;
            _CrtMemCheckpoint (&ms2);
            _CrtMemState diff;

            if (_CrtMemDifference (&diff, &ms1, &ms2))
            {
                _CrtDumpMemoryLeaks ();
                throw "Memory leaks detected";
            }
        }

        return 0;
    }
    catch (const char *e)
    {
        cerr << "Exception: " << e << endl;

        return -1;
    }
}
