/*
    TGA: Targa file operations

    05/01/97 JSP Creation
*/

#ifndef TGA_H
#define TGA_H

/*
**  FILE STRUCTURE FOR THE ORIGINAL TRUEVISION TGA FILE
**      FIELD 1 :    NUMBER OF CHARACTERS IN ID FIELD (1 BYTES)
**      FIELD 2 :    COLOR MAP TYPE (1 BYTES)
**      FIELD 3 :    IMAGE TYPE CODE (1 BYTES)
**                    = 0    NO IMAGE DATA INCLUDED
**                    = 1    UNCOMPRESSED, COLOR-MAPPED IMAGE
**                    = 2    UNCOMPRESSED, TRUE-COLOR IMAGE
**                    = 3    UNCOMPRESSED, BLACK AND WHITE IMAGE
**                    = 9    RUN-LENGTH ENCODED COLOR-MAPPED IMAGE
**                    = 10 RUN-LENGTH ENCODED TRUE-COLOR IMAGE
**                    = 11 RUN-LENGTH ENCODED BLACK AND WHITE IMAGE
**      FIELD 4 :    COLOR MAP SPECIFICATION    (5 BYTES)
**                4.1 : COLOR MAP ORIGIN (2 BYTES)
**                4.2 : COLOR MAP LENGTH (2 BYTES)
**                4.3 : COLOR MAP ENTRY SIZE (1 BYTES)
**      FIELD 5 :    IMAGE SPECIFICATION (10 BYTES)
**                5.1 : X-ORIGIN OF IMAGE (2 BYTES)
**                5.2 : Y-ORIGIN OF IMAGE (2 BYTES)
**                5.3 : WIDTH OF IMAGE (2 BYTES)
**                5.4 : HEIGHT OF IMAGE (2 BYTES)
**                5.5 : IMAGE PIXEL SIZE (1 BYTE)
**                5.6 : IMAGE DESCRIPTOR BYTE (1 BYTE) 
**                      BITS 0 - 3 :    ATTRIBUTE BITS PER PIXEL
**                      BIT  4 :        LEFT TO RIGHT PIXEL ORDERING
**                      BIT  5 :        TOP TO BOTTOM PIXEL ORDERING
**                      BIT  6 :        0
**                      BIT  7 :        0
**      FIELD 6 :    IMAGE ID FIELD (LENGTH SPECIFIED BY FIELD 1)
**      FIELD 7 :    COLOR MAP DATA (BIT WIDTH SPECIFIED BY FIELD 4.3 AND
**                NUMBER OF COLOR MAP ENTRIES SPECIFIED IN FIELD 4.2)
**      FIELD 8 :    IMAGE DATA FIELD (WIDTH AND HEIGHT SPECIFIED IN
**                FIELD 5.3 AND 5.4)
**
**      ALL BYTE ORDERING IS LITTLE-ENDIAN
*/

struct TGAHeader
{
    unsigned char id_field_length;
    unsigned char color_map_type;
    unsigned char image_type_code;
    unsigned short color_map_origin;
    unsigned short color_map_length;
    unsigned char color_map_entry_size;
    unsigned short x_origin;
    unsigned short y_origin;
    unsigned short width;
    unsigned short height;
    unsigned char bits_per_pixel;
    unsigned char image_descriptor;
};

// Image descriptor byte masks

enum TGAImageDescriptorMask
{
    LEFT_TO_RIGHT_MASK = 0x10,
    TOP_TO_BOTTOM_MASK = 0x20,
    INTERLEAVE_MASK = 0xC0
};

enum TGAImageTypeCode
{
    TGA_COLOR_MAPPED = 1,
    TGA_TRUE_COLOR = 2,
    TGA_GREY_SCALE = 3
};

class TGA
{
    public:
        TGA ();
        ~TGA ();
        void Load (const char *filename);
        void Load (unsigned width, 
            unsigned height, 
            unsigned bits_per_pixel, 
            unsigned image_type_code,
            const void *image);
        void Save (const char *filename);
        unsigned Width () { return _tga_header.width; }
        unsigned Height () { return _tga_header.height; }
        void *Image () { return _image; }
        unsigned ImageTypeCode () { return _tga_header.image_type_code; }
        unsigned BitsPerPixel () { return _tga_header.bits_per_pixel; }
        unsigned BytesPerPixel () { return (_tga_header.bits_per_pixel + 7) / 8; }

    private:
        unsigned char *_image;
        TGAHeader _tga_header;
};

#endif
